//+------------------------------------------------------------------+
//|                                                       RsiBar.mqh |
//|                                                      nicholishen |
//|                            http://www.reddit.com/u/nicholishenFX |
//+------------------------------------------------------------------+
#property copyright "nicholishen"
#property link      "http://www.reddit.com/u/nicholishenFX"
#property version   "1.00"
#property strict
#include <stdlib.mqh>
#include <Arrays\List.mqh>
#include <ChartObjects\ChartObjectsLines.mqh>
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CRates : public CObject
{
public:
   MqlRates rates[];
   CRates()
   {
      ArraySetAsSeries(rates,true);
      CopyRates(NULL,0,0,Bars,rates);
   }
};
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CRsiBar : public CObject
{
private:
   MqlRates          m_rsi_bar;
   CRates           *m_rates;
   CChartObjectTrend m_h2l;
   CChartObjectTrend m_o2c;
   int               m_rsi_period;
public:
   CRsiBar(int i,int rsiPeriod,CRates *rates):m_rsi_period(rsiPeriod)
   {
      m_rates=rates;
      int window = WindowFind("RSIBars");
      if(window<0)
      {
         Print("Couldn't find subwindow: ",ErrorDescription(GetLastError()));
         return;
      }  
      m_rsi_bar.time  = m_rates.rates[i].time;
      string name  = "__RsiBar__"+string(m_rsi_bar.time);
      m_h2l.Create(0,name+"1",window,m_rsi_bar.time,0,m_rsi_bar.time,0);    
      m_o2c.Create(0,name+"2",window,m_rsi_bar.time,0,m_rsi_bar.time,0);
      m_o2c.Width(3);
      m_h2l.RayRight(false);
      m_o2c.RayRight(false);
      Update(m_rates,i);
   }
   void  Update(CRates *rates,int i=0)
   {
      m_rates = rates;
      m_rsi_bar.open    = Rsi(PRICE_OPEN,i);
      m_rsi_bar.high    = Rsi(PRICE_HIGH,i);
      m_rsi_bar.low     = Rsi(PRICE_LOW,i);
      m_rsi_bar.close   = Rsi(PRICE_CLOSE,i);
      color col = m_rsi_bar.open < m_rsi_bar.close ? clrLimeGreen : clrRed;
      m_h2l.Color(col);
      m_o2c.Color(col);
      m_h2l.Price(0,m_rsi_bar.high);
      m_h2l.Price(1,m_rsi_bar.low);
      m_o2c.Price(0,m_rsi_bar.open);
      m_o2c.Price(1,m_rsi_bar.close);
      string tt = "O-"+DoubleToString(m_rsi_bar.open,2)+" H-"+DoubleToString(m_rsi_bar.high,2)
                +" L-"+DoubleToString(m_rsi_bar.low,2)+" C-"+DoubleToString(m_rsi_bar.close,2);
      m_h2l.Tooltip(tt);
      m_o2c.Tooltip(tt);
   }
private:
   double Rsi(const int applied,const int i)
   {
      #define POS_BUFFER 1
      #define NEG_BUFFER 2
      double rsi=0;
      double prev_pos = iCustom(Symbol(),Period(),"RSI",m_rsi_period,POS_BUFFER,i+1);
      double prev_neg = iCustom(Symbol(),Period(),"RSI",m_rsi_period,NEG_BUFFER,i+1);
      double diff=0;
      if(applied==PRICE_CLOSE)
         diff=m_rates.rates[i].close - m_rates.rates[i+1].close;
      else 
      if(applied==PRICE_OPEN)
         diff=m_rates.rates[i].open - m_rates.rates[i+1].close;
      else 
      if(applied==PRICE_HIGH)
         diff=m_rates.rates[i].high - m_rates.rates[i+1].close;
      else 
      if(applied==PRICE_LOW)
         diff=m_rates.rates[i].low - m_rates.rates[i+1].close;  
         
      double curr_pos = (prev_pos*(m_rsi_period-1)+(diff>0.0?diff:0.0))/m_rsi_period;
      double curr_neg = (prev_neg*(m_rsi_period-1)+(diff<0.0?-diff:0.0))/m_rsi_period;
      if(curr_neg!=0.0)
         rsi=100.0-100.0/(1+curr_pos/curr_neg);
      else
      {
         if(curr_pos!=0.0)
            rsi=100.0;
         else
            rsi=50.0;
      }
      return rsi;
   }
};
